/*
** nislib.h              NIS+ client access function definitions
**
** Copyright (c) 1993 Signum Support AB, Sweden
**
** This file is part of the NYS Library.
**
** The NYS Library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public License as
** published by the Free Software Foundation; either version 2 of the
** License, or (at your option) any later version.
**
** The NYS Library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
** 
** You should have received a copy of the GNU Library General Public
** License along with the NYS Library; see the file COPYING.LIB.  If
** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
** Cambridge, MA 02139, USA.
**
** Author: Peter Eriksson <pen@signum.se>
*/

#ifndef __NISLIB_NISLIB_H__
#define __NISLIB_NISLIB_H__

/*
** Standard NIS+ client access functions
*/
extern nis_result *nis_lookup(nis_name name, u_long flags);
extern nis_result *nis_first_entry(nis_name name);
extern nis_result *nis_next_entry(nis_name name, netobj *key);
extern void nis_freeresult(nis_result *res);

extern nis_name nis_local_directory(void);
extern nis_name nis_local_host(void);
extern nis_name nis_local_group(void);
extern nis_name nis_local_principal(void);
extern nis_name *nis_getnames(const nis_name name);

extern const char *nis_sperrno(const nis_error status);
extern void nis_perror(const nis_error status, const char *label);
extern void nis_lerror(const nis_error status, const char *label);
extern char *nis_sperror(const nis_error status, const char *label);


/*
** Defines for the nis_lookup() and nis_list()
*/
#define FOLLOW_LINKS     0x01
#define FOLLOW_PATH      0x02
#define HARD_LOOKUP      0x04
#define ALL_RESULTS      0x08
#define NO_CACHE         0x10
#define MASTER_ONLY      0x20
#define EXPAND_NAME      0x40

/*
** NYS-specific NIS+ client functions, you should
** probably not use these in your programs if you care
** at all for portability
*/
extern int nis_setup(void);

#endif

